==> mybatix-spring-boot-starter-3.0.4-sources.jar: org/mybatis/spring/boot/autoconfigure/MybatisAutoConfiguration.java <==
package org.mybatis.spring.boot.autoconfigure;

import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.mapping.DatabaseIdProvider;
import org.apache.ibatis.plugin.Interceptor;
import org.apache.ibatis.scripting.LanguageDriver;
import org.apache.ibatis.session.Configuration;
import org.apache.ibatis.session.ExecutorType;
import org.apache.ibatis.session.SqlSessionFactory;
import org.apache.ibatis.type.TypeHandler;
import org.mybatis.spring.SqlSessionFactoryBean;
import org.mybatis.spring.SqlSessionTemplate;
import org.mybatis.spring.mapper.MapperFactoryBean;
import org.mybatis.spring.mapper.MapperScannerConfigurer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanWrapper;
import org.springframework.beans.BeanWrapperImpl;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.beans.factory.BeanFactoryAware;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.ObjectProvider;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.BeanDefinitionRegistry;
import org.springframework.boot.autoconfigure.AutoConfigurationPackages;
import org.springframework.boot.autoconfigure.AutoConfigureAfter;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.autoconfigure.condition.ConditionalOnSingleCandidate;
import org.springframework.boot.autoconfigure.jdbc.DataSourceAutoConfiguration;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Import;
import org.springframework.context.annotation.ImportBeanDefinitionRegistrar;
import org.springframework.context.annotation.Primary;
import org.springframework.core.io.Resource;
import org.springframework.core.io.ResourceLoader;
import org.springframework.core.type.AnnotationMetadata;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Nullable;
import javax.sql.DataSource;
import java.beans.PropertyDescriptor;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import io.polaris.framework.core.context.bean.PlatformAnnotationBeanNameGenerator;

/**
 * {@link EnableAutoConfiguration Auto-Configuration} for Mybatis. Contributes a {@link SqlSessionFactory} and a
 * {@link SqlSessionTemplate}.
 * <p>
 * If {@link org.mybatis.spring.annotation.MapperScan} is used, or a configuration file is specified as a property,
 * those will be considered, otherwise this auto-configuration will attempt to register mappers based on the interface
 * definitions in or under the root auto-configuration package.
 *
 *
 * 由于在Spring事务中通过{@link org.mybatis.spring.SqlSessionUtils#getSqlSession(SqlSessionFactory) }方法获取{@link org.apache.ibatis.session.SqlSession}，不支持对相同{@link SqlSessionFactory}下的连接进行{@link ExecutorType}模式切换，所以此处自动构建两个{@link SqlSessionFactory}，一为默认模式（通常为{@link ExecutorType#SIMPLE}），一为{@link ExecutorType#BATCH}模式
 *
 * @author Eddú Meléndez
 * @author Josh Long
 * @author Kazuki Shimizu
 * @author Eduardo Macarrón
 * @author Qt
 * @since 1.8,  Dec 26, 2023
 */
@org.springframework.context.annotation.Configuration
@ConditionalOnClass({SqlSessionFactory.class, SqlSessionFactoryBean.class})
@ConditionalOnSingleCandidate(DataSource.class)
@EnableConfigurationProperties(MybatisProperties.class)
@AutoConfigureAfter({DataSourceAutoConfiguration.class, MybatisLanguageDriverAutoConfiguration.class})
public class MybatisAutoConfiguration {

	private static final Logger logger = LoggerFactory.getLogger(MybatisAutoConfiguration.class);


	@ConditionalOnMissingBean({SqlSessionFactory.class})
	@org.springframework.context.annotation.Configuration
	@ConditionalOnClass({SqlSessionFactory.class, SqlSessionFactoryBean.class})
	@ConditionalOnSingleCandidate(DataSource.class)
	@EnableConfigurationProperties(MybatisProperties.class)
	@AutoConfigureAfter({DataSourceAutoConfiguration.class, MybatisLanguageDriverAutoConfiguration.class})
	public static class SqlSessionFactoryConfiguration implements InitializingBean {
		private final MybatisProperties properties;
		private final Interceptor[] interceptors;
		private final TypeHandler[] typeHandlers;
		private final LanguageDriver[] languageDrivers;
		private final ResourceLoader resourceLoader;
		private final DatabaseIdProvider databaseIdProvider;
		private final List<ConfigurationCustomizer> configurationCustomizers;

		public SqlSessionFactoryConfiguration(MybatisProperties properties, ObjectProvider<Interceptor[]> interceptorsProvider,
			ObjectProvider<TypeHandler[]> typeHandlersProvider, ObjectProvider<LanguageDriver[]> languageDriversProvider,
			ResourceLoader resourceLoader, ObjectProvider<DatabaseIdProvider> databaseIdProvider,
			ObjectProvider<List<ConfigurationCustomizer>> configurationCustomizersProvider) {
			this.properties = properties;
			this.interceptors = interceptorsProvider.getIfAvailable();
			this.typeHandlers = typeHandlersProvider.getIfAvailable();
			this.languageDrivers = languageDriversProvider.getIfAvailable();
			this.resourceLoader = resourceLoader;
			this.databaseIdProvider = databaseIdProvider.getIfAvailable();
			this.configurationCustomizers = configurationCustomizersProvider.getIfAvailable();
		}

		@Override
		public void afterPropertiesSet() {
			checkConfigFileExists();
		}

		private void checkConfigFileExists() {
			if (this.properties.isCheckConfigLocation() && StringUtils.hasText(this.properties.getConfigLocation())) {
				Resource resource = this.resourceLoader.getResource(this.properties.getConfigLocation());
				Assert.state(resource.exists(),
					"Cannot find config location: " + resource + " (please add config file or check your Mybatis configuration)");
			}
		}

		@Bean
		@Primary
		public SqlSessionFactory sqlSessionFactory(DataSource dataSource) throws Exception {
			return buildSqlSessionFactory(dataSource);
		}

		@Bean
		public SqlSessionFactory batchSqlSessionFactory(DataSource dataSource) throws Exception {
			return new SqlSessionFactoryProxy(buildSqlSessionFactory(dataSource), ExecutorType.BATCH);
		}

		@Nullable
		private SqlSessionFactory buildSqlSessionFactory(DataSource dataSource) throws Exception {
			SqlSessionFactoryBean factory = new SqlSessionFactoryBean();
			factory.setDataSource(dataSource);
			factory.setVfs(SpringBootVFS.class);
			if (StringUtils.hasText(this.properties.getConfigLocation())) {
				factory.setConfigLocation(this.resourceLoader.getResource(this.properties.getConfigLocation()));
			}
			applyConfiguration(factory);
			if (this.properties.getConfigurationProperties() != null) {
				factory.setConfigurationProperties(this.properties.getConfigurationProperties());
			}
			if (!ObjectUtils.isEmpty(this.interceptors)) {
				factory.setPlugins(this.interceptors);
			}
			if (this.databaseIdProvider != null) {
				factory.setDatabaseIdProvider(this.databaseIdProvider);
			}
			if (StringUtils.hasLength(this.properties.getTypeAliasesPackage())) {
				factory.setTypeAliasesPackage(this.properties.getTypeAliasesPackage());
			}
			if (this.properties.getTypeAliasesSuperType() != null) {
				factory.setTypeAliasesSuperType(this.properties.getTypeAliasesSuperType());
			}
			if (StringUtils.hasLength(this.properties.getTypeHandlersPackage())) {
				factory.setTypeHandlersPackage(this.properties.getTypeHandlersPackage());
			}
			if (!ObjectUtils.isEmpty(this.typeHandlers)) {
				factory.setTypeHandlers(this.typeHandlers);
			}
			if (!ObjectUtils.isEmpty(this.properties.resolveMapperLocations())) {
				factory.setMapperLocations(this.properties.resolveMapperLocations());
			}
			Set<String> factoryPropertyNames = Stream
				.of(new BeanWrapperImpl(SqlSessionFactoryBean.class).getPropertyDescriptors()).map(PropertyDescriptor::getName)
				.collect(Collectors.toSet());
			Class<? extends LanguageDriver> defaultLanguageDriver = this.properties.getDefaultScriptingLanguageDriver();
			if (factoryPropertyNames.contains("scriptingLanguageDrivers") && !ObjectUtils.isEmpty(this.languageDrivers)) {
				// Need to mybatis-spring 2.0.2+
				factory.setScriptingLanguageDrivers(this.languageDrivers);
				if (defaultLanguageDriver == null && this.languageDrivers.length == 1) {
					defaultLanguageDriver = this.languageDrivers[0].getClass();
				}
			}
			if (factoryPropertyNames.contains("defaultScriptingLanguageDriver")) {
				// Need to mybatis-spring 2.0.2+
				factory.setDefaultScriptingLanguageDriver(defaultLanguageDriver);
			}
			return factory.getObject();
		}

		private void applyConfiguration(SqlSessionFactoryBean factory) {
			Configuration configuration = this.properties.getConfiguration();
			if (configuration == null && !StringUtils.hasText(this.properties.getConfigLocation())) {
				configuration = new Configuration();
			}
			if (configuration != null && !CollectionUtils.isEmpty(this.configurationCustomizers)) {
				for (ConfigurationCustomizer customizer : this.configurationCustomizers) {
					customizer.customize(configuration);
				}
			}
			factory.setConfiguration(configuration);
		}

	}

	@ConditionalOnMissingBean({SqlSessionTemplate.class})
	@org.springframework.context.annotation.Configuration
	@ConditionalOnClass({SqlSessionFactory.class, SqlSessionFactoryBean.class})
	@ConditionalOnSingleCandidate(DataSource.class)
	@EnableConfigurationProperties(MybatisProperties.class)
	@AutoConfigureAfter({DataSourceAutoConfiguration.class, MybatisLanguageDriverAutoConfiguration.class})
	public static class SqlSessionTemplateConfiguration {
		private final MybatisProperties properties;

		public SqlSessionTemplateConfiguration(MybatisProperties properties) {
			this.properties = properties;
		}

		@Bean
		@Primary
		public SqlSessionTemplate sqlSessionTemplate(SqlSessionFactory sqlSessionFactory) {
			ExecutorType executorType = this.properties.getExecutorType();
			if (executorType != null) {
				return new SqlSessionTemplate(sqlSessionFactory, executorType);
			} else {
				return new SqlSessionTemplate(sqlSessionFactory);
			}
		}

		@Bean
		public SqlSessionTemplate batchSqlSessionTemplate(@Qualifier("batchSqlSessionFactory") SqlSessionFactory batchSqlSessionFactory) {
			return new SqlSessionTemplate(batchSqlSessionFactory, ExecutorType.BATCH);
		}

	}


	/**
	 * This will just scan the same base package as Spring Boot does. If you want more power, you can explicitly use
	 * {@link org.mybatis.spring.annotation.MapperScan} but this will get typed mappers working correctly, out-of-the-box,
	 * similar to using Spring Data JPA repositories.
	 */
	public static class AutoConfiguredMapperScannerRegistrar implements BeanFactoryAware, ImportBeanDefinitionRegistrar {

		private BeanFactory beanFactory;

		@Override
		public void registerBeanDefinitions(AnnotationMetadata importingClassMetadata, BeanDefinitionRegistry registry) {

			if (!AutoConfigurationPackages.has(this.beanFactory)) {
				logger.debug("Could not determine auto-configuration package, automatic mapper scanning disabled.");
				return;
			}

			logger.debug("Searching for mappers annotated with @Mapper");

			List<String> packages = AutoConfigurationPackages.get(this.beanFactory);
			if (logger.isDebugEnabled()) {
				packages.forEach(pkg -> logger.debug("Using auto-configuration base package '{}'", pkg));
			}

			BeanDefinitionBuilder builder = BeanDefinitionBuilder.genericBeanDefinition(MapperScannerConfigurer.class);
			builder.addPropertyValue("processPropertyPlaceHolders", true);
			builder.addPropertyValue("annotationClass", Mapper.class);
			builder.addPropertyValue("nameGenerator", PlatformAnnotationBeanNameGenerator.INSTANCE);
			builder.addPropertyValue("basePackage", StringUtils.collectionToCommaDelimitedString(packages));
			BeanWrapper beanWrapper = new BeanWrapperImpl(MapperScannerConfigurer.class);
			Set<String> propertyNames = Stream.of(beanWrapper.getPropertyDescriptors()).map(PropertyDescriptor::getName)
				.collect(Collectors.toSet());
			if (propertyNames.contains("lazyInitialization")) {
				// Need to mybatis-spring 2.0.2+
				builder.addPropertyValue("lazyInitialization", "${mybatis.lazy-initialization:false}");
			}
			if (propertyNames.contains("defaultScope")) {
				// Need to mybatis-spring 2.0.6+
				builder.addPropertyValue("defaultScope", "${mybatis.mapper-default-scope:}");
			}
			registry.registerBeanDefinition(MapperScannerConfigurer.class.getName(), builder.getBeanDefinition());
		}

		@Override
		public void setBeanFactory(BeanFactory beanFactory) {
			this.beanFactory = beanFactory;
		}

	}

	/**
	 * If mapper registering configuration or mapper scanning configuration not present, this configuration allow to scan
	 * mappers based on the same component-scanning path as Spring Boot itself.
	 */
	@org.springframework.context.annotation.Configuration
	@Import(AutoConfiguredMapperScannerRegistrar.class)
	@ConditionalOnMissingBean({MapperFactoryBean.class, MapperScannerConfigurer.class})
	public static class MapperScannerRegistrarNotFoundConfiguration implements InitializingBean {

		@Override
		public void afterPropertiesSet() {
			logger.debug(
				"Not found configuration for registering mapper bean using @MapperScan, MapperFactoryBean and MapperScannerConfigurer.");
		}

	}

}

==> mybatix-spring-boot-starter-3.0.12-sources.jar: org/mybatis/spring/boot/autoconfigure/MybatisAutoConfiguration.java <==
package org.mybatis.spring.boot.autoconfigure;

import java.beans.PropertyDescriptor;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.annotation.Nullable;
import javax.sql.DataSource;

import io.polaris.framework.core.context.bean.PlatformAnnotationBeanNameGenerator;
import io.polaris.mybatis.interceptor.MybatisInterceptors;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.mapping.DatabaseIdProvider;
import org.apache.ibatis.plugin.Interceptor;
import org.apache.ibatis.scripting.LanguageDriver;
import org.apache.ibatis.session.Configuration;
import org.apache.ibatis.session.ExecutorType;
import org.apache.ibatis.session.SqlSessionFactory;
import org.apache.ibatis.type.TypeHandler;
import org.mybatis.spring.SqlSessionFactoryBean;
import org.mybatis.spring.SqlSessionTemplate;
import org.mybatis.spring.mapper.MapperFactoryBean;
import org.mybatis.spring.mapper.MapperScannerConfigurer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanWrapper;
import org.springframework.beans.BeanWrapperImpl;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.beans.factory.BeanFactoryAware;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.ObjectProvider;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.BeanDefinitionRegistry;
import org.springframework.boot.autoconfigure.AutoConfigurationPackages;
import org.springframework.boot.autoconfigure.AutoConfigureAfter;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.autoconfigure.condition.ConditionalOnSingleCandidate;
import org.springframework.boot.autoconfigure.jdbc.DataSourceAutoConfiguration;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Import;
import org.springframework.context.annotation.ImportBeanDefinitionRegistrar;
import org.springframework.context.annotation.Primary;
import org.springframework.core.io.Resource;
import org.springframework.core.io.ResourceLoader;
import org.springframework.core.type.AnnotationMetadata;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

/**
 * {@link EnableAutoConfiguration Auto-Configuration} for Mybatis. Contributes a {@link SqlSessionFactory} and a
 * {@link SqlSessionTemplate}.
 * <p>
 * If {@link org.mybatis.spring.annotation.MapperScan} is used, or a configuration file is specified as a property,
 * those will be considered, otherwise this auto-configuration will attempt to register mappers based on the interface
 * definitions in or under the root auto-configuration package.
 * <p>
 * <p>
 * 由于在Spring事务中通过{@link org.mybatis.spring.SqlSessionUtils#getSqlSession(SqlSessionFactory) }方法获取{@link org.apache.ibatis.session.SqlSession}，不支持对相同{@link SqlSessionFactory}下的连接进行{@link ExecutorType}模式切换，所以此处自动构建两个{@link SqlSessionFactory}，一为默认模式（通常为{@link ExecutorType#SIMPLE}），一为{@link ExecutorType#BATCH}模式
 *
 * @author Eddú Meléndez
 * @author Josh Long
 * @author Kazuki Shimizu
 * @author Eduardo Macarrón
 * @author Qt
 * @since 1.8,  Dec 26, 2023
 */
@org.springframework.context.annotation.Configuration
@ConditionalOnClass({SqlSessionFactory.class, SqlSessionFactoryBean.class})
@ConditionalOnSingleCandidate(DataSource.class)
@EnableConfigurationProperties(MybatisProperties.class)
@AutoConfigureAfter({DataSourceAutoConfiguration.class, MybatisLanguageDriverAutoConfiguration.class})
public class MybatisAutoConfiguration {

	private static final Logger logger = LoggerFactory.getLogger(MybatisAutoConfiguration.class);


	@ConditionalOnMissingBean({SqlSessionFactory.class})
	@org.springframework.context.annotation.Configuration
	@ConditionalOnClass({SqlSessionFactory.class, SqlSessionFactoryBean.class})
	@ConditionalOnSingleCandidate(DataSource.class)
	@EnableConfigurationProperties(MybatisProperties.class)
	@AutoConfigureAfter({DataSourceAutoConfiguration.class, MybatisLanguageDriverAutoConfiguration.class})
	public static class SqlSessionFactoryConfiguration implements InitializingBean {
		private final MybatisProperties properties;
		private final Interceptor[] interceptors;
		private final TypeHandler[] typeHandlers;
		private final LanguageDriver[] languageDrivers;
		private final ResourceLoader resourceLoader;
		private final DatabaseIdProvider databaseIdProvider;
		private final List<ConfigurationCustomizer> configurationCustomizers;

		public SqlSessionFactoryConfiguration(MybatisProperties properties, ObjectProvider<Interceptor[]> interceptorsProvider,
			ObjectProvider<TypeHandler[]> typeHandlersProvider, ObjectProvider<LanguageDriver[]> languageDriversProvider,
			ResourceLoader resourceLoader, ObjectProvider<DatabaseIdProvider> databaseIdProvider,
			ObjectProvider<List<ConfigurationCustomizer>> configurationCustomizersProvider) {
			this.properties = properties;
			this.interceptors = interceptorsProvider.getIfAvailable();
			this.typeHandlers = typeHandlersProvider.getIfAvailable();
			this.languageDrivers = languageDriversProvider.getIfAvailable();
			this.resourceLoader = resourceLoader;
			this.databaseIdProvider = databaseIdProvider.getIfAvailable();
			this.configurationCustomizers = configurationCustomizersProvider.getIfAvailable();
		}

		@Override
		public void afterPropertiesSet() {
			checkConfigFileExists();
		}

		private void checkConfigFileExists() {
			if (this.properties.isCheckConfigLocation() && StringUtils.hasText(this.properties.getConfigLocation())) {
				Resource resource = this.resourceLoader.getResource(this.properties.getConfigLocation());
				Assert.state(resource.exists(),
					"Cannot find config location: " + resource + " (please add config file or check your Mybatis configuration)");
			}
		}

		@Bean
		@Primary
		public SqlSessionFactory sqlSessionFactory(DataSource dataSource) throws Exception {
			return buildSqlSessionFactory(dataSource);
		}

		@Bean
		public SqlSessionFactory batchSqlSessionFactory(DataSource dataSource) throws Exception {
			return new SqlSessionFactoryProxy(sqlSessionFactory(dataSource), ExecutorType.BATCH);
		}

		@Nullable
		private SqlSessionFactory buildSqlSessionFactory(DataSource dataSource) throws Exception {
			SqlSessionFactoryBean factory = new SqlSessionFactoryBean();
			factory.setDataSource(dataSource);
			factory.setVfs(SpringBootVFS.class);
			if (StringUtils.hasText(this.properties.getConfigLocation())) {
				factory.setConfigLocation(this.resourceLoader.getResource(this.properties.getConfigLocation()));
			}
			applyConfiguration(factory);
			if (this.properties.getConfigurationProperties() != null) {
				factory.setConfigurationProperties(this.properties.getConfigurationProperties());
			}
			if (!ObjectUtils.isEmpty(this.interceptors)) {
				factory.setPlugins(this.interceptors);
			}
			if (this.databaseIdProvider != null) {
				factory.setDatabaseIdProvider(this.databaseIdProvider);
			}
			if (StringUtils.hasLength(this.properties.getTypeAliasesPackage())) {
				factory.setTypeAliasesPackage(this.properties.getTypeAliasesPackage());
			}
			if (this.properties.getTypeAliasesSuperType() != null) {
				factory.setTypeAliasesSuperType(this.properties.getTypeAliasesSuperType());
			}
			if (StringUtils.hasLength(this.properties.getTypeHandlersPackage())) {
				factory.setTypeHandlersPackage(this.properties.getTypeHandlersPackage());
			}
			if (!ObjectUtils.isEmpty(this.typeHandlers)) {
				factory.setTypeHandlers(this.typeHandlers);
			}
			if (!ObjectUtils.isEmpty(this.properties.resolveMapperLocations())) {
				factory.setMapperLocations(this.properties.resolveMapperLocations());
			}
			Set<String> factoryPropertyNames = Stream
				.of(new BeanWrapperImpl(SqlSessionFactoryBean.class).getPropertyDescriptors()).map(PropertyDescriptor::getName)
				.collect(Collectors.toSet());
			Class<? extends LanguageDriver> defaultLanguageDriver = this.properties.getDefaultScriptingLanguageDriver();
			if (factoryPropertyNames.contains("scriptingLanguageDrivers") && !ObjectUtils.isEmpty(this.languageDrivers)) {
				// Need to mybatis-spring 2.0.2+
				factory.setScriptingLanguageDrivers(this.languageDrivers);
				if (defaultLanguageDriver == null && this.languageDrivers.length == 1) {
					defaultLanguageDriver = this.languageDrivers[0].getClass();
				}
			}
			if (factoryPropertyNames.contains("defaultScriptingLanguageDriver")) {
				// Need to mybatis-spring 2.0.2+
				factory.setDefaultScriptingLanguageDriver(defaultLanguageDriver);
			}
			return factory.getObject();
		}

		private void applyConfiguration(SqlSessionFactoryBean factory) {
			Configuration configuration = this.properties.getConfiguration();
			if (configuration == null && !StringUtils.hasText(this.properties.getConfigLocation())) {
				configuration = new Configuration();
			}
			if (configuration != null && !CollectionUtils.isEmpty(this.configurationCustomizers)) {
				for (ConfigurationCustomizer customizer : this.configurationCustomizers) {
					customizer.customize(configuration);
				}
			}
			factory.setConfiguration(configuration);
		}

	}

	@ConditionalOnMissingBean({SqlSessionTemplate.class})
	@org.springframework.context.annotation.Configuration
	@ConditionalOnClass({SqlSessionFactory.class, SqlSessionFactoryBean.class})
	@ConditionalOnSingleCandidate(DataSource.class)
	@EnableConfigurationProperties(MybatisProperties.class)
	@AutoConfigureAfter({DataSourceAutoConfiguration.class, MybatisLanguageDriverAutoConfiguration.class})
	public static class SqlSessionTemplateConfiguration {
		private final MybatisProperties properties;

		public SqlSessionTemplateConfiguration(MybatisProperties properties) {
			this.properties = properties;
		}

		@Bean
		@Primary
		public SqlSessionTemplate sqlSessionTemplate(SqlSessionFactory sqlSessionFactory) {
			ExecutorType executorType = this.properties.getExecutorType();
			if (executorType != null) {
				return new SqlSessionTemplate(sqlSessionFactory, executorType);
			} else {
				return new SqlSessionTemplate(sqlSessionFactory);
			}
		}

		@Bean
		public SqlSessionTemplate batchSqlSessionTemplate(@Qualifier("batchSqlSessionFactory") SqlSessionFactory batchSqlSessionFactory) {
			/*
			 * mapper = batchSqlSessionTemplate.getMapper(XxxMapper.class);
			 * mapper.doXxx();
			 * batchSqlSessionTemplate.flushStatements();
			 */
			return new SqlSessionTemplate(batchSqlSessionFactory, ExecutorType.BATCH);
		}

	}


	/**
	 * This will just scan the same base package as Spring Boot does. If you want more power, you can explicitly use
	 * {@link org.mybatis.spring.annotation.MapperScan} but this will get typed mappers working correctly, out-of-the-box,
	 * similar to using Spring Data JPA repositories.
	 */
	public static class AutoConfiguredMapperScannerRegistrar implements BeanFactoryAware, ImportBeanDefinitionRegistrar {

		private BeanFactory beanFactory;

		@Override
		public void registerBeanDefinitions(AnnotationMetadata importingClassMetadata, BeanDefinitionRegistry registry) {

			if (!AutoConfigurationPackages.has(this.beanFactory)) {
				logger.debug("Could not determine auto-configuration package, automatic mapper scanning disabled.");
				return;
			}

			logger.debug("Searching for mappers annotated with @Mapper");

			List<String> packages = AutoConfigurationPackages.get(this.beanFactory);
			if (logger.isDebugEnabled()) {
				packages.forEach(pkg -> logger.debug("Using auto-configuration base package '{}'", pkg));
			}

			BeanDefinitionBuilder builder = BeanDefinitionBuilder.genericBeanDefinition(MapperScannerConfigurer.class);
			builder.addPropertyValue("processPropertyPlaceHolders", true);
			builder.addPropertyValue("annotationClass", Mapper.class);
			builder.addPropertyValue("nameGenerator", PlatformAnnotationBeanNameGenerator.INSTANCE);
			builder.addPropertyValue("basePackage", StringUtils.collectionToCommaDelimitedString(packages));
			BeanWrapper beanWrapper = new BeanWrapperImpl(MapperScannerConfigurer.class);
			Set<String> propertyNames = Stream.of(beanWrapper.getPropertyDescriptors()).map(PropertyDescriptor::getName)
				.collect(Collectors.toSet());
			if (propertyNames.contains("lazyInitialization")) {
				// Need to mybatis-spring 2.0.2+
				builder.addPropertyValue("lazyInitialization", "${mybatis.lazy-initialization:false}");
			}
			if (propertyNames.contains("defaultScope")) {
				// Need to mybatis-spring 2.0.6+
				builder.addPropertyValue("defaultScope", "${mybatis.mapper-default-scope:}");
			}
			registry.registerBeanDefinition(MapperScannerConfigurer.class.getName(), builder.getBeanDefinition());
		}

		@Override
		public void setBeanFactory(BeanFactory beanFactory) {
			this.beanFactory = beanFactory;
		}

	}

	/**
	 * If mapper registering configuration or mapper scanning configuration not present, this configuration allow to scan
	 * mappers based on the same component-scanning path as Spring Boot itself.
	 */
	@org.springframework.context.annotation.Configuration
	@Import(AutoConfiguredMapperScannerRegistrar.class)
	@ConditionalOnMissingBean({MapperFactoryBean.class, MapperScannerConfigurer.class})
	public static class MapperScannerRegistrarNotFoundConfiguration implements InitializingBean {

		@Override
		public void afterPropertiesSet() {
			logger.debug(
				"Not found configuration for registering mapper bean using @MapperScan, MapperFactoryBean and MapperScannerConfigurer.");
		}

	}

}
